import { Observable } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { Find, SbxCore } from 'sbxcorejs';
export declare class SbxCoreService extends SbxCore {
    httpClient: HttpClient;
    private headers;
    constructor(httpClient: HttpClient);
    initialize(domain: number, appKey: string, baseUrl?: string): void;
    addHeaderAttr(name: string, value: string): void;
    removeHeaderAttr(name: string): void;
    private getHeaders();
    getHeadersJSON(): any;
    $p(path: string): string;
    /**
     * AUTH
     */
    /**
     * @param {string} token
     */
    validate(token: string): Promise<any>;
    /**
     * @param {string} token
     * @return {Observable<any>}
     */
    validateRx(token: string): Observable<any>;
    /**
     *
     * @param {string} login
     * @param {string} email
     * @param {string} name
     * @param {string} password
     */
    signUp(login: string, email: string, name: string, password: string): Promise<any>;
    /**
     * @param {string} login
     * @param {string} email
     * @param {string} name
     * @param {string} password
     * @return {Observable<any>}
     */
    signUpRx(login: string, email: string, name: string, password: string): Observable<any>;
    /**
     * @param {string} login
     * @param {string} password
     * @param {Callback} callBack
     * @param {domain}
     */
    login(login: string, password: string, domain?: number): Promise<any>;
    /**
     * @param {string} login
     * @param {string} password
     *  @param {domain}
     * @return {Observable<any>}
     */
    loginRx(login: string, password: string, domain?: number): Observable<any>;
    /**
     * Send email to changePassword
     * @param {string} userEmail
     * @param {string} subject
     * @param {string} emailTemplate
     */
    sendPasswordRequest(userEmail: string, subject: string, emailTemplate: string): Promise<any>;
    /**
     * Send email to changePassword
     * @param {string} userEmail
     * @param {string} subject
     * @param {string} emailTemplate
     * @return {Observable<Object>}
     */
    sendPasswordRequestRx(userEmail: string, subject: string, emailTemplate: string): Observable<any>;
    /**
     * change password with email code
     * @param {number} userId
     * @param {number} userCode
     * @param {string} newPassword
     */
    requestChangePassword(userId: any, userCode: any, newPassword: any): Promise<Object>;
    /**
     * change password with email code
     * @param {number} userId
     * @param {number} userCode
     * @param {string} newPassword
     * @return {Observable<Object>}
     */
    requestChangePasswordRx(userId: any, userCode: any, newPassword: any): Observable<Object>;
    /**
     * change password
     * @param {string} newPassword
     */
    changePassword(newPassword: any): Promise<Object>;
    /**
     * change password
     * @param {string} newPassword
     * @return {Observable<Object>}
     */
    changePasswordRx(newPassword: any): Observable<Object>;
    /***
     * DATA
     */
    /**
     * @param {string} model the name model in sbxcloud
     * @param data can be a JSON, or TypeScript Class or Array of both
     *  @param letNull let null data
     */
    insert(model: string, data: any, letNull?: Boolean): Promise<any>;
    /**
     * @param {string} model the name model in sbxcloud
     * @param data can be a JSON, or TypeScript Class or Array of both
     * @param letNull let null data
     */
    update(model: string, data: any, letNull?: Boolean): Promise<any>;
    /**
     * @param {string} model the name model in sbxcloud
     * @param data can be a JSON, or TypeScript Class or Array of both
     * @param letNull let null data
     * @return {Observable}
     */
    insertRx(model: string, data: any, letNull?: Boolean): Observable<any>;
    /**
     * @param {string} model the name model in sbxcloud
     * @param data can be a JSON, or TypeScript Class or Array of both
     * @param letNull let null data
     * @return {Observable}
     */
    updateRx(model: string, data: any, letNull?: Boolean): Observable<any>;
    /**
     * @param {string} model the name model in sbxcloud
     */
    with(model: string): AngularFind;
    /**
     * @param {EmailData} data
     */
    sendEmail(data: EmailData): Promise<any>;
    /**
     * @param {EmailData} data
     * @return {Observable<any>}
     */
    sendEmailRx(data: EmailData): Observable<any>;
    /**
     * @param data
     */
    paymentCustomer(data: Object): Promise<any>;
    /**
     * @param data
     * @return {Observable<any>}
     */
    paymentCustomerRx(data: Object): Observable<any>;
    /**
     * @param data
     */
    paymentCard(data: Object): Promise<any>;
    /**
     * @param data
     * @return {Observable<any>}
     */
    paymentCardRx(data: Object): Observable<any>;
    /**
     * @param {string} key
     * @param file
     * @return {Observable<any>}
     */
    uploadFileRx(key: string, file: any): Observable<any>;
    /**
     *
     * @param {string} key
     * @param file
     */
    uploadFile(key: string, file: any): Promise<any>;
    /**
     * @param {string} key
     * @return {Observable<any>}
     */
    downloadFileRx(key: string): Observable<any>;
    /**
     *
     * @param {string} key
     */
    downloadFile(key: string): Promise<any>;
    /**
     * CLOUDSCRIPT
     */
    /**
     *
     * @param {string} key
     * @param params
     * @return {Observable<any>}
     */
    runRx(key: string, params: any): Observable<any>;
    /**
     * @param {string} key
     * @param params
     */
    run(key: string, params: any): Promise<any>;
}
export declare class SbxResponse<T> {
    success: boolean;
    message?: string;
    results?: Array<T>;
    fetched_results?: any;
    total_pages: number;
    isAny(): boolean;
    toAny(): any;
}
export declare class AngularFind extends Find {
    private core;
    private url;
    private totalpages;
    private isFind;
    constructor(model: string, core: SbxCoreService);
    delete(): Promise<any>;
    deleteRx(): Observable<any>;
    find(toFetch: string[]): any;
    find<T>(): any;
    /**
     * @param {Array} toFetch Optional params to auto map fetches result.
     * match T object, no return fetched_results
     */
    findRx<T>(toFetch?: any[]): Observable<Array<T> | any>;
    /**
   * @param {Array} toFetch Optional params to auto map fetches result.
   */
    loadAll(toFetch?: any[]): Promise<{
        success: boolean;
        results: any[];
        fetched_results: {};
    }>;
    /**
   * @param {Array} toFetch Optional params to auto map fetches result.
   */
    loadAllRx(toFetch?: any[]): Observable<{
        success: boolean;
        results: any[];
        fetched_results: {};
    }>;
    /**
     * Change the url, to find or to delete
     * @param isFind if true, the url is gotten from urls.find else urls.delete
     */
    private setUrl(isFind);
    /**
     * get the data
     * @param {any[]} toFetch Optional params to auto map fetches result.
     * @return {Observable<any>}
     */
    private thenRx<T>(toFetch?);
    /**
     * Is used to paginate load all
     * @param query
     * @return {Observable<any>}
     */
    private findPage(query?);
}
export interface EmailData {
    subject: string;
    from: string;
    template?: string;
    template_key?: string;
    data?: any;
    to: string | Array<string>;
    bcc?: string | Array<string>;
    cc?: string | Array<string>;
}
