var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { merge as observableMerge, of as observableOf } from 'rxjs';
import { toArray, mergeAll, map, mergeMap } from 'rxjs/operators';
import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Find, SbxCore } from 'sbxcorejs';
var SbxCoreService = /** @class */ (function (_super) {
    __extends(SbxCoreService, _super);
    function SbxCoreService(httpClient) {
        var _this = _super.call(this) || this;
        _this.httpClient = httpClient;
        return _this;
    }
    SbxCoreService.prototype.initialize = function (domain, appKey, baseUrl) {
        if (baseUrl === void 0) { baseUrl = 'https://sbxcloud.com/api'; }
        _super.prototype.initialize.call(this, domain, appKey, baseUrl);
        this.headers = new HttpHeaders()
            .set('App-Key', appKey);
    };
    SbxCoreService.prototype.addHeaderAttr = function (name, value) {
        this.headers = this.getHeaders().set(name, value);
    };
    SbxCoreService.prototype.removeHeaderAttr = function (name) {
        this.headers = this.getHeaders().delete(name);
    };
    SbxCoreService.prototype.getHeaders = function () {
        return this.headers;
    };
    SbxCoreService.prototype.getHeadersJSON = function () {
        return this.getHeaders().append('Content-Type', 'application/json');
    };
    SbxCoreService.prototype.$p = function (path) {
        return SbxCore.environment.baseUrl + path;
    };
    /**
     * AUTH
     */
    /**
     * @param {string} token
     */
    SbxCoreService.prototype.validate = function (token) {
        return this.validateRx(token).toPromise();
    };
    /**
     * @param {string} token
     * @return {Observable<any>}
     */
    SbxCoreService.prototype.validateRx = function (token) {
        var httpParams = new HttpParams().set('token', token);
        var option = { headers: this.getHeadersJSON(), params: httpParams };
        return this.httpClient.get(this.$p(this.urls.validate), option).pipe(map(function (data) { return data; }));
    };
    /**
     *
     * @param {string} login
     * @param {string} email
     * @param {string} name
     * @param {string} password
     */
    SbxCoreService.prototype.signUp = function (login, email, name, password) {
        return this.signUpRx(login, email, name, password).toPromise();
    };
    /**
     * @param {string} login
     * @param {string} email
     * @param {string} name
     * @param {string} password
     * @return {Observable<any>}
     */
    SbxCoreService.prototype.signUpRx = function (login, email, name, password) {
        if (this.validateLogin(login) && this.validateEmail(email)) {
            var option = { headers: this.getHeadersJSON() };
            var params = '?email=' + this.encodeEmails(email) + '&password=' + encodeURIComponent(password) + '&name='
                + name + '&login=' + login + '&domain=' + SbxCore.environment.domain.toLocaleString();
            return this.httpClient.get(this.$p(this.urls.register) + params, option).pipe(map(function (data) { return data; }));
        }
        else {
            return observableOf({ success: false,
                error: 'Login or email contains invalid characters. Letters, numbers and underscore are accepted' });
        }
    };
    /**
     * @param {string} login
     * @param {string} password
     * @param {Callback} callBack
     * @param {domain}
     */
    SbxCoreService.prototype.login = function (login, password, domain) {
        return this.loginRx(login, password, domain).toPromise();
    };
    /**
     * @param {string} login
     * @param {string} password
     *  @param {domain}
     * @return {Observable<any>}
     */
    SbxCoreService.prototype.loginRx = function (login, password, domain) {
        if ((this.validateLogin(login) && login.indexOf('@') < 0) || (login.indexOf('@') >= 0 && this.validateEmail(login))) {
            var option = { headers: this.getHeadersJSON() };
            var params = '?login=' + this.encodeEmails(login) + '&password=' + encodeURIComponent(password)
                + (domain ? '&domain=' + domain : '');
            return this.httpClient.get(this.$p(this.urls.login) + params, option).pipe(map(function (data) { return data; }));
        }
        else {
            return observableOf({ success: false,
                error: 'Login contains invalid characters. Letters, numbers and underscore are accepted' });
        }
    };
    /**
     * Send email to changePassword
     * @param {string} userEmail
     * @param {string} subject
     * @param {string} emailTemplate
     */
    SbxCoreService.prototype.sendPasswordRequest = function (userEmail, subject, emailTemplate) {
        return this.sendPasswordRequestRx(userEmail, subject, emailTemplate).toPromise();
    };
    /**
     * Send email to changePassword
     * @param {string} userEmail
     * @param {string} subject
     * @param {string} emailTemplate
     * @return {Observable<Object>}
     */
    SbxCoreService.prototype.sendPasswordRequestRx = function (userEmail, subject, emailTemplate) {
        var body = { user_email: userEmail,
            domain: SbxCore.environment.domain, subject: subject, email_template: emailTemplate };
        var option = { headers: this.getHeadersJSON() };
        return this.httpClient.post(this.$p(this.urls.password), body, option).pipe(map(function (data) { return data; }));
    };
    /**
     * change password with email code
     * @param {number} userId
     * @param {number} userCode
     * @param {string} newPassword
     */
    SbxCoreService.prototype.requestChangePassword = function (userId, userCode, newPassword) {
        return this.requestChangePasswordRx(userId, userCode, newPassword).toPromise();
    };
    /**
     * change password with email code
     * @param {number} userId
     * @param {number} userCode
     * @param {string} newPassword
     * @return {Observable<Object>}
     */
    SbxCoreService.prototype.requestChangePasswordRx = function (userId, userCode, newPassword) {
        var body = { domain: SbxCore.environment.domain,
            password: newPassword,
            user_id: userId,
            code: userCode };
        var option = { headers: this.getHeadersJSON() };
        return this.httpClient.put(this.$p(this.urls.password), body, option).pipe(map(function (data) { return data; }));
    };
    /**
     * change password
     * @param {string} newPassword
     */
    SbxCoreService.prototype.changePassword = function (newPassword) {
        return this.changePasswordRx(newPassword).toPromise();
    };
    /**
     * change password
     * @param {string} newPassword
     * @return {Observable<Object>}
     */
    SbxCoreService.prototype.changePasswordRx = function (newPassword) {
        var httpParams = new HttpParams().set('domain', SbxCore.environment.domain).set('password', newPassword);
        var option = { headers: this.getHeadersJSON(), params: httpParams };
        return this.httpClient.get(this.$p(this.urls.update_password), option).pipe(map(function (data) { return data; }));
    };
    /***
     * DATA
     */
    /**
     * @param {string} model the name model in sbxcloud
     * @param data can be a JSON, or TypeScript Class or Array of both
     *  @param letNull let null data
     */
    SbxCoreService.prototype.insert = function (model, data, letNull) {
        return this.insertRx(model, data, letNull).toPromise();
    };
    /**
     * @param {string} model the name model in sbxcloud
     * @param data can be a JSON, or TypeScript Class or Array of both
     * @param letNull let null data
     */
    SbxCoreService.prototype.update = function (model, data, letNull) {
        return this.updateRx(model, data, letNull).toPromise();
    };
    /**
     * @param {string} model the name model in sbxcloud
     * @param data can be a JSON, or TypeScript Class or Array of both
     * @param letNull let null data
     * @return {Observable}
     */
    SbxCoreService.prototype.insertRx = function (model, data, letNull) {
        var body = this.upsert(model, data, letNull);
        var option = { headers: this.getHeadersJSON() };
        return this.httpClient.post(this.$p(this.urls.row), body, option).pipe(map(function (res) { return res; }));
    };
    /**
     * @param {string} model the name model in sbxcloud
     * @param data can be a JSON, or TypeScript Class or Array of both
     * @param letNull let null data
     * @return {Observable}
     */
    SbxCoreService.prototype.updateRx = function (model, data, letNull) {
        var body = this.upsert(model, data, letNull);
        var option = { headers: this.getHeadersJSON() };
        return this.httpClient.post(this.$p(this.urls.update), body, option).pipe(map(function (res) { return res; }));
    };
    /**
     * @param {string} model the name model in sbxcloud
     */
    SbxCoreService.prototype.with = function (model) {
        return new AngularFind(model, this);
    };
    /**
     * @param {EmailData} data
     */
    SbxCoreService.prototype.sendEmail = function (data) {
        return this.sendEmailRx(data).toPromise();
    };
    /**
     * @param {EmailData} data
     * @return {Observable<any>}
     */
    SbxCoreService.prototype.sendEmailRx = function (data) {
        var mail = {
            subject: data.subject,
            to: data.to,
            domain: SbxCore.environment.domain,
            from: data.from,
        };
        if (data.template) {
            mail.html = data.template;
        }
        else {
            mail.template_key = data.template_key;
        }
        if (data.cc) {
            mail.cc = data.cc;
        }
        if (data.bcc) {
            mail.bcc = data.bcc;
        }
        if (data.data) {
            mail.data = data.data;
        }
        var option = { headers: this.getHeadersJSON() };
        return this.httpClient.post(this.$p(this.urls.send_mail), mail, option).pipe(map(function (res) { return res; }));
    };
    /**
     * @param data
     */
    SbxCoreService.prototype.paymentCustomer = function (data) {
        return this.paymentCustomerRx(data).toPromise();
    };
    /**
     * @param data
     * @return {Observable<any>}
     */
    SbxCoreService.prototype.paymentCustomerRx = function (data) {
        data['domain'] = SbxCore.environment.domain;
        var option = { headers: this.getHeadersJSON() };
        return this.httpClient.post(this.$p(this.urls.payment_customer), data, option).pipe(map(function (res) { return res; }));
    };
    /**
     * @param data
     */
    SbxCoreService.prototype.paymentCard = function (data) {
        return this.paymentCardRx(data).toPromise();
    };
    /**
     * @param data
     * @return {Observable<any>}
     */
    SbxCoreService.prototype.paymentCardRx = function (data) {
        data['domain'] = SbxCore.environment.domain;
        var option = { headers: this.getHeadersJSON() };
        return this.httpClient.post(this.$p(this.urls.payment_card), data, option).pipe(map(function (res) { return res; }));
    };
    /**
     * @param {string} key
     * @param file
     * @return {Observable<any>}
     */
    SbxCoreService.prototype.uploadFileRx = function (key, file) {
        var input = new FormData();
        input.append('file', file);
        input.append('model', JSON.stringify({ key: key }));
        var option = { headers: this.getHeaders() };
        return this.httpClient.post(this.$p(this.urls.uploadFile), input, option).pipe(map(function (res) { return res; }));
    };
    /**
     *
     * @param {string} key
     * @param file
     */
    SbxCoreService.prototype.uploadFile = function (key, file) {
        return this.uploadFileRx(key, file).toPromise();
    };
    /**
     * @param {string} key
     * @return {Observable<any>}
     */
    SbxCoreService.prototype.downloadFileRx = function (key) {
        var httpParams = new HttpParams().set('action', 'download').set('key', key);
        var option = { headers: this.getHeaders(), params: httpParams };
        return this.httpClient.get(this.$p(this.urls.downloadFile), option).pipe(map(function (res) { return res; }));
    };
    /**
     *
     * @param {string} key
     */
    SbxCoreService.prototype.downloadFile = function (key) {
        return this.downloadFileRx(key).toPromise();
    };
    /**
     * CLOUDSCRIPT
     */
    /**
     *
     * @param {string} key
     * @param params
     * @return {Observable<any>}
     */
    SbxCoreService.prototype.runRx = function (key, params) {
        var option = { headers: this.getHeadersJSON() };
        return this.httpClient.post(this.$p(this.urls.cloudscript_run), { key: key, params: params }, option).pipe(map(function (res) { return res; }));
    };
    /**
     * @param {string} key
     * @param params
     */
    SbxCoreService.prototype.run = function (key, params) {
        return this.runRx(key, params).toPromise();
    };
    SbxCoreService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    SbxCoreService.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    return SbxCoreService;
}(SbxCore));
export { SbxCoreService };
var SbxResponse = /** @class */ (function () {
    function SbxResponse() {
    }
    SbxResponse.prototype.isAny = function () {
        return this.results === undefined
            || this.results === null
            || this.results.length === 0
            || this.results[0].constructor.name === 'Object';
    };
    SbxResponse.prototype.toAny = function () {
        return this;
    };
    return SbxResponse;
}());
export { SbxResponse };
var AngularFind = /** @class */ (function (_super) {
    __extends(AngularFind, _super);
    function AngularFind(model, core) {
        var _this = _super.call(this, model, SbxCore.environment.domain) || this;
        _this.core = core;
        _this.totalpages = 1;
        return _this;
    }
    AngularFind.prototype.delete = function () {
        return this.deleteRx().toPromise();
    };
    AngularFind.prototype.deleteRx = function () {
        this.setUrl(false);
        return this.thenRx();
    };
    /**
    * @param {Array} toFetch Optional params to auto map fetches result.
    * match T object, no return fetched_results
     */
    AngularFind.prototype.find = function (toFetch) {
        if (toFetch === void 0) { toFetch = []; }
        return this.findRx(toFetch).toPromise();
    };
    /**
     * @param {Array} toFetch Optional params to auto map fetches result.
     * match T object, no return fetched_results
     */
    AngularFind.prototype.findRx = function (toFetch) {
        if (toFetch === void 0) { toFetch = []; }
        this.setUrl(true);
        return this.thenRx(toFetch);
    };
    /**
   * @param {Array} toFetch Optional params to auto map fetches result.
   */
    AngularFind.prototype.loadAll = function (toFetch) {
        if (toFetch === void 0) { toFetch = []; }
        return this.loadAllRx(toFetch).toPromise();
    };
    /**
   * @param {Array} toFetch Optional params to auto map fetches result.
   */
    AngularFind.prototype.loadAllRx = function (toFetch) {
        var _this = this;
        if (toFetch === void 0) { toFetch = []; }
        this.setPageSize(100);
        var query = this.query.compile();
        return this.thenRx().pipe(mergeMap(function (response) {
            _this.totalpages = response.total_pages;
            var i = 2;
            var temp = [observableOf(response)];
            while (i <= _this.totalpages) {
                var queryAux = JSON.parse(JSON.stringify(query));
                queryAux.page = i;
                temp.push(_this.findPage(queryAux));
                i = i + 1;
            }
            return observableMerge(temp).pipe(mergeAll(5), toArray());
        }), map(function (res) { return res; }), map(function (results) {
            var result = [];
            var fetched_results = {};
            results.forEach(function (array) {
                var v = array;
                result = result.concat(v.results);
                if (v.fetched_results) {
                    var objs = Object.keys(v.fetched_results);
                    for (var i = 0; i < objs.length; i++) {
                        var type_name = objs[i];
                        if (!fetched_results.hasOwnProperty(type_name)) {
                            fetched_results[type_name] = {};
                        }
                        var keys = Object.keys(v.fetched_results[type_name]);
                        for (var j = 0; j < keys.length; j++) {
                            var key = keys[j];
                            if (v.fetched_results[type_name].hasOwnProperty(key)) {
                                fetched_results[type_name][key] = v.fetched_results[type_name][key];
                            }
                        }
                    }
                    if (toFetch.length) {
                        result = _this.core.mapFetchesResult(result, toFetch);
                    }
                }
            });
            return { success: true, results: result, fetched_results: fetched_results };
        }));
    };
    /**
     * Change the url, to find or to delete
     * @param isFind if true, the url is gotten from urls.find else urls.delete
     */
    AngularFind.prototype.setUrl = function (isFind) {
        this.url = isFind ? this.core.$p(this.core.urls.find) : this.core.$p(this.core.urls.delete);
    };
    /**
     * get the data
     * @param {any[]} toFetch Optional params to auto map fetches result.
     * @return {Observable<any>}
     */
    AngularFind.prototype.thenRx = function (toFetch) {
        var _this = this;
        if (toFetch === void 0) { toFetch = []; }
        var option = { headers: this.core.getHeadersJSON() };
        return this.core.httpClient.post(this.url, this.query.compile(), option).pipe(map(function (res) {
            if (res instanceof SbxResponse && res.isAny()) {
                var newRest = res.toAny();
                if (toFetch.length && _this.isFind) {
                    return _this.core.mapFetchesResult(newRest, toFetch);
                }
                return newRest;
            }
            else {
                if (res instanceof SbxResponse) {
                    if (!res.success) {
                        throw new Error(res.message);
                    }
                    else {
                        return res.results;
                    }
                }
                else {
                    return res;
                }
            }
        }));
    };
    /**
     * Is used to paginate load all
     * @param query
     * @return {Observable<any>}
     */
    AngularFind.prototype.findPage = function (query) {
        var option = { headers: this.core.getHeadersJSON() };
        return this.core.httpClient.post(this.core.$p(this.core.urls.find), (query == null) ? this.query.compile() : query, option).pipe(map(function (res) { return res; }));
    };
    return AngularFind;
}(Find));
export { AngularFind };
//# sourceMappingURL=sbxcore.service.js.map