import { map } from 'rxjs/operators';
import { Injectable } from '@angular/core';
import { SbxCoreService } from '../sbxcore.service';
import { CookieService } from 'ngx-cookie-service';
var SbxSessionService = /** @class */ (function () {
    function SbxSessionService(sbxCoreService, cookieService) {
        this.sbxCoreService = sbxCoreService;
        this.cookieService = cookieService;
        this.daysToExpire = 30;
        this.cookieToken = 'token';
    }
    SbxSessionService.prototype.initialize = function (domain, appKey) {
        this.sbxCoreService.initialize(domain, appKey);
        this.islogged();
    };
    /**
     * Initialize service with environment
     * @param environment (domain, base_url, appkey)
     */
    SbxSessionService.prototype.initializeWithEnvironment = function (environment) {
        this.sbxCoreService.initialize(environment.domain, environment.appKey, environment.baseUrl);
        this.islogged();
    };
    /**
     * General User methods
     */
    SbxSessionService.prototype.getCurrentUser = function () {
        return (this._user == null) ? this._user = new User() : this._user;
    };
    SbxSessionService.prototype.islogged = function () {
        this.loadCookieToken();
        if (this.getCurrentUser().token != null) {
            this.sbxCoreService.addHeaderAttr('Authorization', 'Bearer ' + this.getCurrentUser().token);
            return true;
        }
        else {
            return false;
        }
    };
    /**
     * methods that uses cookies
     */
    SbxSessionService.prototype.loadCookieToken = function () {
        this.getCurrentUser().token = this.cookieService.check(this.cookieToken) ? this.cookieService.get(this.cookieToken) : null;
    };
    SbxSessionService.prototype.updateCookieToken = function (token) {
        var today = new Date().getTime();
        this.cookieService.set(this.cookieToken, token, new Date(today + this.daysToExpire * SbxSessionService.day));
    };
    SbxSessionService.prototype.updateUser = function (data) {
        this.getCurrentUser().token = data.token;
        this.getCurrentUser().id = data.user.id;
        this.getCurrentUser().name = data.user.name;
        this.getCurrentUser().login = data.user.login;
        this.getCurrentUser().email = data.user.email;
        this.sbxCoreService.addHeaderAttr('Authorization', 'Bearer ' + data.token);
    };
    /**
     * Auth user methods
     */
    SbxSessionService.prototype.login = function (login, password, domain) {
        return this.loginRx(login, password, domain).toPromise();
    };
    SbxSessionService.prototype.loginRx = function (login, password, domain) {
        var _this = this;
        return this.sbxCoreService.loginRx(login, password, domain).pipe(map(function (data) {
            if (data.success) {
                _this.updateUser(data);
            }
            return data;
        }));
    };
    SbxSessionService.prototype.validate = function (token) {
        this.validateRx(token).toPromise();
    };
    SbxSessionService.prototype.validateRx = function (token) {
        var _this = this;
        return this.sbxCoreService.validateRx(token).pipe(map(function (data) {
            if (data.success) {
                data.token = token;
                _this.updateUser(data);
            }
            return data;
        }));
    };
    SbxSessionService.prototype.logout = function () {
        this.cookieService.delete(this.cookieToken);
        this.sbxCoreService.removeHeaderAttr('Authorization');
        this._user = null;
    };
    SbxSessionService.prototype.signUp = function (login, email, name, password) {
        return this.signUpRx(login, email, name, password).toPromise();
    };
    SbxSessionService.prototype.signUpRx = function (login, email, name, password) {
        var _this = this;
        return this.sbxCoreService.signUpRx(login, email, name, password).pipe(map(function (data) {
            if (data.success) {
                _this.updateUser(data);
            }
            return data;
        }));
    };
    SbxSessionService.day = 86400000;
    SbxSessionService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    SbxSessionService.ctorParameters = function () { return [
        { type: SbxCoreService },
        { type: CookieService }
    ]; };
    return SbxSessionService;
}());
export { SbxSessionService };
var User = /** @class */ (function () {
    function User() {
    }
    Object.defineProperty(User.prototype, "name", {
        get: function () {
            return this._name;
        },
        set: function (value) {
            this._name = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(User.prototype, "token", {
        get: function () {
            return this._token;
        },
        set: function (value) {
            this._token = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(User.prototype, "id", {
        get: function () {
            return this._id;
        },
        set: function (value) {
            this._id = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(User.prototype, "email", {
        get: function () {
            return this._email;
        },
        set: function (value) {
            this._email = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(User.prototype, "login", {
        get: function () {
            return this._login;
        },
        set: function (value) {
            this._login = value;
        },
        enumerable: true,
        configurable: true
    });
    return User;
}());
export { User };
//# sourceMappingURL=sbxsession.service.js.map